"""
Functions and routines associated with Enasis Network Homie Automate.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from typing import Literal



HomieChildKinds = Literal[
    'origin',
    'device',
    'group',
    'scene',
    'aspire',
    'desire']
